@@ -48,15 +48,14 @@ NULL
 #' There is a `plot()` method, which shows the probabilities of all predicted
 #' distributions, however, only if the probability is greater than zero.
 #'
-#' @examples
-#' if (require("lme4") && require("parameters") &&
-#'   require("see") && require("patchwork") && require("randomForest")) {
-#'   data(sleepstudy)
+#' @examplesIf require("lme4") && require("parameters") && require("randomForest")
+#' data(sleepstudy, package = "lme4")
+#' model <<- lme4::lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
+#' check_distribution(model)
+#'
+#' @examplesIf require("see") && require("patchwork") && require("randomForest")
+#' plot(check_distribution(model))
 #'
-#'   model <<- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
-#'   check_distribution(model)
-#'   plot(check_distribution(model))
-#' }
 #' @export
 check_distribution <- function(model) {
   UseMethod("check_distribution")
@@ -78,7 +77,6 @@ check_distribution.default <- function(model) {
   } else {
     x <- stats::residuals(model)
   }
-  # x_scaled <- .normalize(x)
   dat <- .extract_features(x)
 
   dist_residuals <- as.data.frame(t(stats::predict(classify_distribution, dat, type = "prob")))
@@ -104,7 +102,7 @@ check_distribution.default <- function(model) {
 
   class(out) <- unique(c("check_distribution", "see_check_distribution", class(out)))
   attr(out, "data") <- model
-  attr(out, "object_name") <- deparse(substitute(model), width.cutoff = 500)
+  attr(out, "object_name") <- insight::safe_deparse_symbol(substitute(model))
 
   out
 }
@@ -172,11 +170,11 @@ check_distribution.numeric <- function(model) {
   insight::check_if_installed("randomForest")
 
   dat <- .extract_features(model)
-  dist <- as.data.frame(t(stats::predict(classify_distribution, dat, type = "prob")))
+  distance <- as.data.frame(t(stats::predict(classify_distribution, dat, type = "prob")))
 
   out <- data.frame(
-    Distribution = rownames(dist),
-    p_Vector = dist[[1]],
+    Distribution = rownames(distance),
+    p_Vector = distance[[1]],
     stringsAsFactors = FALSE,
     row.names = NULL
   )
@@ -192,24 +190,27 @@ check_distribution.numeric <- function(model) {
 # utilities -----------------------------
 
 .extract_features <- function(x) {
+  # validation check, remove missings
+  x <- x[!is.na(x)]
+
   data.frame(
-    "SD" = stats::sd(x),
-    "MAD" = stats::mad(x, constant = 1),
-    "Mean_Median_Distance" = mean(x) - stats::median(x),
-    "Mean_Mode_Distance" = mean(x) - as.numeric(bayestestR::map_estimate(x, bw = "nrd0")),
-    "SD_MAD_Distance" = stats::sd(x) - stats::mad(x, constant = 1),
-    "Var_Mean_Distance" = stats::var(x) - mean(x),
-    "Range_SD" = diff(range(x)) / stats::sd(x),
-    "Range" = diff(range(x)),
-    "IQR" = stats::IQR(x),
-    "Skewness" = .skewness(x),
-    "Kurtosis" = .kurtosis(x),
-    "Uniques" = length(unique(x)) / length(x),
-    "N_Uniques" = length(unique(x)),
-    "Min" = min(x),
-    "Max" = max(x),
-    "Proportion_Positive" = sum(x >= 0) / length(x),
-    "Integer" = all(.is_integer(x))
+    SD = stats::sd(x),
+    MAD = stats::mad(x, constant = 1),
+    Mean_Median_Distance = mean(x) - stats::median(x),
+    Mean_Mode_Distance = mean(x) - as.numeric(bayestestR::map_estimate(x, bw = "nrd0")),
+    SD_MAD_Distance = stats::sd(x) - stats::mad(x, constant = 1),
+    Var_Mean_Distance = stats::var(x) - mean(x),
+    Range_SD = diff(range(x)) / stats::sd(x),
+    Range = diff(range(x)),
+    IQR = stats::IQR(x),
+    Skewness = .skewness(x),
+    Kurtosis = .kurtosis(x),
+    Uniques = length(unique(x)) / length(x),
+    N_Uniques = length(unique(x)),
+    Min = min(x),
+    Max = max(x),
+    Proportion_Positive = sum(x >= 0) / length(x),
+    Integer = all(.is_integer(x))
   )
 }
 
@@ -217,14 +218,8 @@ check_distribution.numeric <- function(model) {
 
 .is_integer <- function(x) {
   tryCatch(
-    expr = {
-      ifelse(is.infinite(x), FALSE, x %% 1 == 0)
-    },
-    warning = function(w) {
-      is.integer(x)
-    },
-    error = function(e) {
-      FALSE
-    }
+    ifelse(is.infinite(x), FALSE, x %% 1 == 0),
+    warning = function(w) is.integer(x),
+    error = function(e) FALSE
   )
 }
