@@ -32,7 +32,7 @@
 #'   [`r2_nakagawa()`], [`r2_tjur()`], [`r2_xu()`] and
 #'   [`r2_zeroinflated()`].
 #'
-#' @examples
+#' @examplesIf require("lme4")
 #' # Pseudo r-quared for GLM
 #' model <- glm(vs ~ wt + mpg, data = mtcars, family = "binomial")
 #' r2(model)
@@ -41,21 +41,16 @@
 #' model <- lm(mpg ~ wt + hp, data = mtcars)
 #' r2(model, ci = 0.95)
 #'
-#' if (require("lme4")) {
-#'   model <- lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
-#'   r2(model)
-#' }
+#' model <- lme4::lmer(Sepal.Length ~ Petal.Length + (1 | Species), data = iris)
+#' r2(model)
 #' @export
 r2 <- function(model, ...) {
   UseMethod("r2")
 }
 
 
-
-
 # Default models -----------------------------------------------
 
-
 #' @rdname r2
 #' @export
 r2.default <- function(model, ci = NULL, verbose = TRUE, ...) {
@@ -64,7 +59,8 @@ r2.default <- function(model, ci = NULL, verbose = TRUE, ...) {
     return(.r2_ci(model, ci = ci, verbose = verbose, ...))
   }
 
-  if (is.null(minfo <- list(...)$model_info)) {
+  minfo <- list(...)$model_info
+  if (is.null(minfo)) {
     minfo <- suppressWarnings(insight::model_info(model, verbose = FALSE))
   }
 
@@ -113,6 +109,10 @@ r2.lm <- function(model, ci = NULL, ...) {
   .r2_lm(summary(model))
 }
 
+#' @export
+r2.phylolm <- r2.lm
+
+# helper -------------
 
 .r2_lm <- function(model_summary, ci = NULL) {
   out <- list(
@@ -139,7 +139,6 @@ r2.lm <- function(model, ci = NULL, ...) {
 }
 
 
-
 #' @export
 r2.summary.lm <- function(model, ci = NULL, ...) {
   if (!is.null(ci) && !is.na(ci)) {
@@ -149,7 +148,6 @@ r2.summary.lm <- function(model, ci = NULL, ...) {
 }
 
 
-
 #' @export
 r2.systemfit <- function(model, ...) {
   out <- lapply(summary(model)$eq, function(model_summary) {
@@ -172,11 +170,11 @@ r2.systemfit <- function(model, ...) {
 #' @export
 r2.lm_robust <- function(model, ...) {
   out <- list(
-    "R2" = tryCatch(
+    R2 = tryCatch(
       model[["r.squared"]],
       error = function(e) NULL
     ),
-    "R2_adjusted" = tryCatch(
+    R2_adjusted = tryCatch(
       model[["adj.r.squared"]],
       error = function(e) NULL
     )
@@ -197,8 +195,6 @@ r2.ols <- function(model, ...) {
   structure(class = "r2_generic", out)
 }
 
-
-
 #' @export
 r2.lrm <- r2.ols
 
@@ -206,7 +202,6 @@ r2.lrm <- r2.ols
 r2.cph <- r2.ols
 
 
-
 #' @export
 r2.mhurdle <- function(model, ...) {
   resp <- insight::get_response(model, verbose = FALSE)
@@ -229,7 +224,6 @@ r2.mhurdle <- function(model, ...) {
 }
 
 
-
 #' @export
 r2.aov <- function(model, ci = NULL, ...) {
   if (!is.null(ci) && !is.na(ci)) {
@@ -251,7 +245,6 @@ r2.aov <- function(model, ci = NULL, ...) {
 }
 
 
-
 #' @export
 r2.mlm <- function(model, ...) {
   model_summary <- summary(model)
@@ -260,7 +253,7 @@ r2.mlm <- function(model, ...) {
     tmp <- list(
       R2 = model_summary[[i]]$r.squared,
       R2_adjusted = model_summary[[i]]$adj.r.squared,
-      Response = sub("Response ", "", i)
+      Response = sub("Response ", "", i, fixed = TRUE)
     )
     names(tmp$R2) <- "R2"
     names(tmp$R2_adjusted) <- "adjusted R2"
@@ -275,21 +268,21 @@ r2.mlm <- function(model, ...) {
 }
 
 
-
 #' @export
 r2.glm <- function(model, ci = NULL, verbose = TRUE, ...) {
   if (!is.null(ci) && !is.na(ci)) {
     return(.r2_ci(model, ci = ci, verbose = verbose, ...))
   }
 
-  if (is.null(info <- list(...)$model_info)) {
+  info <- list(...)$model_info
+  if (is.null(info)) {
     info <- suppressWarnings(insight::model_info(model, verbose = FALSE))
   }
 
   if (info$family %in% c("gaussian", "inverse.gaussian")) {
     out <- r2.default(model, ...)
   } else if (info$is_logit && info$is_bernoulli) {
-    out <- list("R2_Tjur" = r2_tjur(model, ...))
+    out <- list(R2_Tjur = r2_tjur(model, model_info = info, ...))
     attr(out, "model_type") <- "Logistic"
     names(out$R2_Tjur) <- "Tjur's R2"
     class(out) <- c("r2_pseudo", class(out))
@@ -299,7 +292,7 @@ r2.glm <- function(model, ci = NULL, verbose = TRUE, ...) {
     }
     out <- NULL
   } else {
-    out <- list("R2_Nagelkerke" = r2_nagelkerke(model, ...))
+    out <- list(R2_Nagelkerke = r2_nagelkerke(model, ...))
     names(out$R2_Nagelkerke) <- "Nagelkerke's R2"
     attr(out, "model_type") <- "Generalized Linear"
     class(out) <- c("r2_pseudo", class(out))
@@ -311,6 +304,14 @@ r2.glm <- function(model, ci = NULL, verbose = TRUE, ...) {
 r2.glmx <- r2.glm
 
 
+#' @export
+r2.nestedLogit <- function(model, ci = NULL, verbose = TRUE, ...) {
+  out <- list(R2_Tjur = r2_tjur(model, ...))
+  attr(out, "model_type") <- "Logistic"
+  class(out) <- c("r2_pseudo", class(out))
+  out
+}
+
 
 
 # mfx models ---------------------
@@ -356,7 +357,7 @@ r2.model_fit <- r2.logitmfx
 
 #' @export
 r2.BBreg <- function(model, ...) {
-  out <- list("R2_CoxSnell" = r2_coxsnell(model))
+  out <- list(R2_CoxSnell = r2_coxsnell(model))
   names(out$R2_CoxSnell) <- "Cox & Snell's R2"
   class(out) <- c("r2_pseudo", class(out))
   out
@@ -376,7 +377,7 @@ r2.bayesx <- r2.BBreg
 
 #' @export
 r2.censReg <- function(model, ...) {
-  out <- list("R2_Nagelkerke" = r2_nagelkerke(model))
+  out <- list(R2_Nagelkerke = r2_nagelkerke(model))
   names(out$R2_Nagelkerke) <- "Nagelkerke's R2"
   class(out) <- c("r2_pseudo", class(out))
   out
@@ -394,9 +395,6 @@ r2.clm2 <- r2.censReg
 #' @export
 r2.coxph <- r2.censReg
 
-#' @export
-r2.mclogit <- r2.censReg
-
 #' @export
 r2.polr <- r2.censReg
 
@@ -415,7 +413,15 @@ r2.brmultinom <- r2.censReg
 #' @export
 r2.bife <- r2.censReg
 
+#' @export
+r2.mclogit <- function(model, ...) {
+  r2_nagelkerke(model)
+}
 
+#' @export
+r2.mblogit <- function(model, ...) {
+  r2_nagelkerke(model)
+}
 
 
 
@@ -461,9 +467,6 @@ r2.merMod <- function(model, ci = NULL, tolerance = 1e-5, ...) {
   r2_nakagawa(model, ci = ci, tolerance = tolerance, ...)
 }
 
-#' @export
-r2.glmmTMB <- r2.merMod
-
 #' @export
 r2.cpglmm <- r2.merMod
 
@@ -485,6 +488,48 @@ r2.MixMod <- r2.merMod
 #' @export
 r2.rlmerMod <- r2.merMod
 
+#' @export
+r2.glmmTMB <- function(model, ci = NULL, tolerance = 1e-5, verbose = TRUE, ...) {
+  # most models are mixed models
+  if (insight::is_mixed_model(model)) {
+    return(r2_nakagawa(model, ci = ci, tolerance = tolerance, ...))
+  } else {
+    if (!is.null(ci) && !is.na(ci)) {
+      return(.r2_ci(model, ci = ci, ...))
+    }
+    # calculate r2 for non-mixed glmmTMB models here -------------------------
+    info <- insight::model_info(model, verbose = FALSE)
+
+    if (info$is_linear) {
+      # for linear models, use the manual calculation
+      out <- .safe(.r2_lm_manual(model))
+    } else if (info$is_logit && info$is_bernoulli) {
+      # logistic regression with binary outcome
+      out <- list(R2_Tjur = r2_tjur(model, model_info = info, ...))
+      attr(out, "model_type") <- "Logistic"
+      names(out$R2_Tjur) <- "Tjur's R2"
+      class(out) <- c("r2_pseudo", class(out))
+    } else if (info$is_binomial && !info$is_bernoulli) {
+      # currently, non-bernoulli binomial models are not supported
+      if (verbose) {
+        insight::format_warning("Can't calculate accurate R2 for binomial models that are not Bernoulli models.")
+      }
+      out <- NULL
+    } else if ((info$is_poisson && !info$is_zero_inflated) || info$is_exponential) {
+      # Poisson-regression or Gamma uses Nagelkerke's R2
+      out <- list(R2_Nagelkerke = r2_nagelkerke(model, ...))
+      names(out$R2_Nagelkerke) <- "Nagelkerke's R2"
+      attr(out, "model_type") <- "Generalized Linear"
+      class(out) <- c("r2_pseudo", class(out))
+    } else if (info$is_zero_inflated) {
+      # zero-inflated models use the default method
+      out <- r2_zeroinflated(model)
+    } else {
+      insight::format_error("`r2()` does not support models of class `glmmTMB` without random effects and this link-function.") # nolint
+    }
+  }
+  out
+}
 
 #' @export
 r2.wbm <- function(model, tolerance = 1e-5, ...) {
@@ -500,8 +545,8 @@ r2.wbm <- function(model, tolerance = 1e-5, ...) {
     names(r2_marginal) <- "Marginal R2"
 
     out <- list(
-      "R2_conditional" = r2_conditional,
-      "R2_marginal" = r2_marginal
+      R2_conditional = r2_conditional,
+      R2_marginal = r2_marginal
     )
 
     attr(out, "model_type") <- "Fixed Effects"
@@ -524,8 +569,8 @@ r2.sem <- function(model, ...) {
   structure(
     class = "r2_nakagawa",
     list(
-      "R2_conditional" = r2_conditional,
-      "R2_marginal" = r2_marginal
+      R2_conditional = r2_conditional,
+      R2_marginal = r2_marginal
     )
   )
 }
@@ -554,14 +599,12 @@ r2.BFBayesFactor <- r2.brmsfit
 #' @export
 r2.gam <- function(model, ...) {
   # gamlss inherits from gam, and summary.gamlss prints results automatically
-  printout <- utils::capture.output(s <- summary(model))
+  printout <- utils::capture.output(s <- summary(model)) # nolint
 
-  if (!is.null(s$r.sq)) {
-    list(
-      R2 = c(`Adjusted R2` = s$r.sq)
-    )
-  } else {
+  if (is.null(s$r.sq)) {
     NextMethod()
+  } else {
+    list(R2 = c(`Adjusted R2` = s$r.sq))
   }
 }
 
@@ -596,7 +639,7 @@ r2.rma <- function(model, ...) {
 #' @export
 r2.feis <- function(model, ...) {
   out <- list(
-    R2 = c(`R2` = model$r2),
+    R2 = c(R2 = model$r2),
     R2_adjusted = c(`adjusted R2` = model$adj.r2)
   )
 
@@ -636,13 +679,18 @@ r2.fixest <- function(model, ...) {
   structure(class = "r2_generic", out)
 }
 
+#' @export
+r2.fixest_multi <- function(model, ...) {
+  lapply(model, r2.fixest)
+}
+
 
 
 #' @export
 r2.felm <- function(model, ...) {
   model_summary <- summary(model)
   out <- list(
-    R2 = c(`R2` = model_summary$r2),
+    R2 = c(R2 = model_summary$r2),
     R2_adjusted = c(`adjusted R2` = model_summary$r2adj)
   )
 
@@ -656,7 +704,7 @@ r2.felm <- function(model, ...) {
 #' @export
 r2.iv_robust <- function(model, ...) {
   out <- list(
-    R2 = c(`R2` = model$r.squared),
+    R2 = c(R2 = model$r.squared),
     R2_adjusted = c(`adjusted R2` = model$adj.r.squared)
   )
 
@@ -670,7 +718,7 @@ r2.iv_robust <- function(model, ...) {
 r2.ivreg <- function(model, ...) {
   model_summary <- summary(model)
   out <- list(
-    R2 = c(`R2` = model_summary$r.squared),
+    R2 = c(R2 = model_summary$r.squared),
     R2_adjusted = c(`adjusted R2` = model_summary$adj.r.squared)
   )
 
@@ -682,7 +730,7 @@ r2.ivreg <- function(model, ...) {
 
 #' @export
 r2.bigglm <- function(model, ...) {
-  out <- list("R2_CoxSnell" = summary(model)$rsq)
+  out <- list(R2_CoxSnell = summary(model)$rsq)
   names(out$R2_CoxSnell) <- "Cox & Snell's R2"
   class(out) <- c("r2_pseudo", class(out))
   out
@@ -692,7 +740,7 @@ r2.bigglm <- function(model, ...) {
 
 #' @export
 r2.biglm <- function(model, ...) {
-  df.int <- ifelse(insight::has_intercept(model), 1, 0)
+  df.int <- as.numeric(insight::has_intercept(model))
   n <- suppressWarnings(insight::n_obs(model))
 
   rsq <- summary(model)$rsq
@@ -716,7 +764,7 @@ r2.biglm <- function(model, ...) {
 r2.lmrob <- function(model, ...) {
   model_summary <- summary(model)
   out <- list(
-    R2 = c(`R2` = model_summary$r.squared),
+    R2 = c(R2 = model_summary$r.squared),
     R2_adjusted = c(`adjusted R2` = model_summary$adj.r.squared)
   )
 
@@ -735,18 +783,12 @@ r2.mmclogit <- function(model, ...) {
 }
 
 
-#' @export
-r2.mclogit <- function(model, ...) {
-  list(R2 = NA)
-}
-
-
 #' @export
 r2.Arima <- function(model, ...) {
-  if (!requireNamespace("forecast", quietly = TRUE)) {
-    list(R2 = NA)
+  if (requireNamespace("forecast", quietly = TRUE)) {
+    list(R2 = stats::cor(stats::fitted(model), insight::get_data(model, verbose = FALSE))^2)
   } else {
-    list(R2 = stats::cor(stats::fitted(model), insight::get_data(model))^2)
+    list(R2 = NA)
   }
 }
 
@@ -756,8 +798,8 @@ r2.Arima <- function(model, ...) {
 r2.plm <- function(model, ...) {
   model_summary <- summary(model)
   out <- list(
-    "R2" = c(`R2` = model_summary$r.squared[1]),
-    "R2_adjusted" = c(`adjusted R2` = model_summary$r.squared[2])
+    R2 = c(R2 = model_summary$r.squared[1]),
+    R2_adjusted = c(`adjusted R2` = model_summary$r.squared[2])
   )
 
   attr(out, "model_type") <- "Panel Data"
@@ -773,8 +815,8 @@ r2.selection <- function(model, ...) {
     return(NULL)
   }
   out <- list(
-    "R2" = c(`R2` = model_summary$rSquared$R2),
-    "R2_adjusted" = c(`adjusted R2` = model_summary$rSquared$R2adj)
+    R2 = c(R2 = model_summary$rSquared$R2),
+    R2_adjusted = c(`adjusted R2` = model_summary$rSquared$R2adj)
   )
 
   attr(out, "model_type") <- "Tobit 2"
@@ -789,7 +831,7 @@ r2.svyglm <- function(model, ...) {
   rsq.adjust <- 1 - ((1 - rsq) * (model$df.null / model$df.residual))
 
   out <- list(
-    R2 = c(`R2` = rsq),
+    R2 = c(R2 = rsq),
     R2_adjusted = c(`adjusted R2` = rsq.adjust)
   )
 
@@ -801,7 +843,7 @@ r2.svyglm <- function(model, ...) {
 
 #' @export
 r2.vglm <- function(model, ...) {
-  out <- list("R2_McKelvey" = r2_mckelvey(model))
+  out <- list(R2_McKelvey = r2_mckelvey(model))
   names(out$R2_McKelvey) <- "McKelvey's R2"
   class(out) <- c("r2_pseudo", class(out))
   out
@@ -814,7 +856,7 @@ r2.vgam <- r2.vglm
 
 #' @export
 r2.DirichletRegModel <- function(model, ...) {
-  out <- list("R2_Nagelkerke" = r2_nagelkerke(model))
+  out <- list(R2_Nagelkerke = r2_nagelkerke(model))
   names(out$R2_Nagelkerke) <- "Nagelkerke's R2"
   class(out) <- c("r2_pseudo", class(out))
   out
@@ -827,15 +869,53 @@ r2.DirichletRegModel <- function(model, ...) {
 # helper -------------------
 
 .check_r2_ci_args <- function(ci = NULL, ci_method = "bootstrap", valid_ci_method = NULL, verbose = TRUE) {
-  if (!is.null(ci) && !is.na(ci)) {
-    if (!is.null(valid_ci_method) && !ci_method %in% valid_ci_method) {
-      if (verbose) {
-        insight::format_warning(
-          paste0("Method `", ci_method, "` to compute confidence intervals for R2 not supported.")
-        )
-      }
-      return(NULL)
+  if (!is.null(ci) && !is.na(ci) && !is.null(valid_ci_method) && !ci_method %in% valid_ci_method) {
+    if (verbose) {
+      insight::format_warning(
+        paste0("Method `", ci_method, "` to compute confidence intervals for R2 not supported.")
+      )
     }
+    return(NULL)
   }
   ci
 }
+
+
+.r2_lm_manual <- function(model) {
+  w <- insight::get_weights(model, verbose = FALSE)
+  r <- stats::residuals(model)
+  f <- stats::fitted(model)
+  n <- length(r)
+  rdf <- .safe(stats::df.residual(model))
+  df_int <- .safe(as.numeric(insight::has_intercept(model)))
+
+  if (insight::has_intercept(model)) {
+    if (is.null(w)) {
+      mss <- sum((f - mean(f))^2)
+    } else {
+      m <- sum(w * f / sum(w))
+      mss <- sum(w * (f - m)^2)
+    }
+  } else if (is.null(w)) {
+    mss <- sum(f^2)
+  } else {
+    mss <- sum(w * f^2)
+  }
+  if (is.null(w)) {
+    rss <- sum(r^2)
+  } else {
+    rss <- sum(w * r^2)
+  }
+  r_squared <- mss / (mss + rss)
+  if (is.null(df_int) || is.null(rdf)) {
+    adj_r2 <- NULL
+  } else {
+    adj_r2 <- 1 - (1 - r_squared) * ((n - df_int) / rdf)
+  }
+  out <- list(R2 = r_squared, R2_adjusted = adj_r2)
+
+  names(out$R2) <- "R2"
+  names(out$R2_adjusted) <- "adjusted R2"
+  attr(out, "model_type") <- "Linear"
+  structure(class = "r2_generic", out)
+}
