@@ -1,7 +1,3 @@
-skip_if_not_installed("bigutilsr")
-skip_if_not_installed("ICS")
-skip_if_not_installed("dbscan")
-
 test_that("zscore negative threshold", {
   expect_error(
     check_outliers(mtcars$mpg, method = "zscore", threshold = -1),
@@ -80,28 +76,38 @@ test_that("mahalanobis which", {
 })
 
 test_that("mahalanobis_robust which", {
+  skip_if_not_installed("bigutilsr")
   expect_identical(
     which(check_outliers(mtcars, method = "mahalanobis_robust", threshold = 25)),
     as.integer(c(7, 9, 21, 24, 27, 28, 29, 31))
   )
 })
 
-## FIXME: Fails on CRAN/windows
-# (should be fixed but not clear why method mcd needs a seed;
-# there should not be an element of randomness to it I think)
 test_that("mcd which", {
+  # (not clear why method mcd needs a seed)
   set.seed(42)
   expect_identical(
-    tail(which(check_outliers(mtcars[1:4], method = "mcd", threshold = 45))),
+    tail(which(check_outliers(mtcars[1:4], method = "mcd", threshold = 45, verbose = FALSE))),
     31L
   )
+  expect_warning(
+    {
+      out <- check_outliers(mtcars, method = "mcd")
+    },
+    regex = "The sample size is too small"
+  )
+  expect_identical(sum(out), 8L)
+  out <- check_outliers(mtcars, method = "mcd", percentage_central = 0.5, verbose = FALSE)
+  expect_identical(sum(out), 15L)
 })
 
 ## FIXME: Fails on CRAN/windows
-# (current CRAN version rstan is not compatible with R > 4.2)
 test_that("ics which", {
-  skip_if_not_installed("rstan", minimum_version = "2.26.0")
-  set.seed(42)
+  # suddenly fails on R Under development (unstable) (2023-09-07 r85102)
+  # gcc-13 (Debian 13.2.0-2) 13.2.0
+  skip_on_cran()
+  skip_if_not_installed("ICS")
+  skip_if_not_installed("ICSOutlier")
   expect_identical(
     which(check_outliers(mtcars, method = "ics", threshold = 0.001)),
     as.integer(c(9, 29))
@@ -109,6 +115,7 @@ test_that("ics which", {
 })
 
 test_that("optics which", {
+  skip_if_not_installed("dbscan")
   expect_identical(
     which(check_outliers(mtcars, method = "optics", threshold = 14)),
     as.integer(c(5, 7, 15, 16, 17, 24, 25, 29, 31))
@@ -116,6 +123,7 @@ test_that("optics which", {
 })
 
 test_that("lof which", {
+  skip_if_not_installed("dbscan")
   expect_identical(
     which(check_outliers(mtcars, method = "lof", threshold = 0.005)),
     31L
@@ -190,6 +198,7 @@ test_that("multiple methods which", {
 
 # We exclude method ics because it is too slow
 test_that("all methods which", {
+  skip_if_not_installed("bigutilsr")
   expect_identical(
     which(check_outliers(mtcars,
       method = c(
@@ -197,11 +206,12 @@ test_that("all methods which", {
         "mahalanobis", "mahalanobis_robust", "mcd", "optics", "lof"
       ),
       threshold = list(
-        "zscore" = 2.2, "zscore_robust" = 2.2, "iqr" = 1.2,
-        "ci" = 0.95, "eti" = 0.95, "hdi" = 0.90, "bci" = 0.95,
-        "mahalanobis" = 20, "mahalanobis_robust" = 25, "mcd" = 25,
-        "optics" = 14, "lof" = 0.005
-      )
+        zscore = 2.2, zscore_robust = 2.2, iqr = 1.2,
+        ci = 0.95, eti = 0.95, hdi = 0.90, bci = 0.95,
+        mahalanobis = 20, mahalanobis_robust = 25, mcd = 25,
+        optics = 14, lof = 0.005
+      ),
+      verbose = FALSE
     )),
     as.integer(c(9, 15, 16, 19, 20, 28, 29, 31))
   )
@@ -211,6 +221,7 @@ test_that("all methods which", {
 
 
 test_that("multiple methods with ID", {
+  skip_if_not_installed("bigutilsr")
   data <- datawizard::rownames_as_column(mtcars, var = "car")
   x <- attributes(check_outliers(data,
     method = c(
@@ -218,12 +229,13 @@ test_that("multiple methods with ID", {
       "mahalanobis", "mahalanobis_robust", "mcd", "optics", "lof"
     ),
     threshold = list(
-      "zscore" = 2.2, "zscore_robust" = 2.2, "iqr" = 1.2,
-      "ci" = 0.95, "eti" = 0.95, "hdi" = 0.90, "bci" = 0.95,
-      "mahalanobis" = 20, "mahalanobis_robust" = 25, "mcd" = 25,
-      "optics" = 14, "lof" = 0.005
+      zscore = 2.2, zscore_robust = 2.2, iqr = 1.2,
+      ci = 0.95, eti = 0.95, hdi = 0.90, bci = 0.95,
+      mahalanobis = 20, mahalanobis_robust = 25, mcd = 25,
+      optics = 14, lof = 0.005
     ),
-    ID = "car"
+    ID = "car",
+    verbose = FALSE
   ))
   expect_identical(
     x$outlier_var$zscore$mpg$car,
@@ -258,6 +270,7 @@ test_that("cook which", {
 # })
 
 test_that("cook multiple methods which", {
+  skip_if_not_installed("dbscan")
   model <- lm(disp ~ mpg + hp, data = mtcars)
   expect_identical(
     which(check_outliers(model, method = c("cook", "optics", "lof"))),
@@ -266,6 +279,7 @@ test_that("cook multiple methods which", {
 })
 
 test_that("pareto which", {
+  skip_if_not_installed("dbscan")
   skip_if_not_installed("rstanarm")
   set.seed(123)
   model <- rstanarm::stan_glm(mpg ~ qsec + wt, data = mtcars, refresh = 0)
@@ -278,6 +292,7 @@ test_that("pareto which", {
 })
 
 test_that("pareto multiple methods which", {
+  skip_if_not_installed("dbscan")
   skip_if_not_installed("rstanarm")
   set.seed(123)
   model <- rstanarm::stan_glm(mpg ~ qsec + wt, data = mtcars, refresh = 0)
@@ -307,7 +322,6 @@ test_that("BayesFactor which", {
 # 7. Next, we test grouped output
 
 test_that("cook multiple methods which", {
-  skip_if_not_installed("datawizard")
   iris2 <- datawizard::data_group(iris, "Species")
   z <- attributes(check_outliers(iris2, method = c("zscore", "iqr")))
   expect_named(
@@ -315,3 +329,16 @@ test_that("cook multiple methods which", {
     c("setosa", "versicolor", "virginica")
   )
 })
+
+
+test_that("check_outliers with invald data", {
+  dd <- data.frame(y = as.difftime(0:5, units = "days"))
+  m1 <- lm(y ~ 1, data = dd)
+  expect_error(
+    expect_message(
+      check_outliers(m1),
+      regex = "Date variables are not supported"
+    ),
+    regex = "No numeric variables found"
+  )
+})
